#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <iostream>
#include <fstream>
#include <stddef.h>
#include <sstream>

#include <omp.h>

#define NR_END 1
#define FREE_ARG char*


using namespace std;

#define SWAP(a,b) {temp=(a);(a)=(b);(b)=temp;}

void nrerror(string napis)
{
    cerr<<napis<<endl;
	exit(1);
}

double *vector(long nl, long nh)
/* allocate a float vector with subscript range v[nl..nh] */
{
	double *v;

	v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl+NR_END;
}

double **matrix(int nrl,int nrh,int ncl,int nch)
{
	int i;
	double **m;

	m=(double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
		if (!m[i]) nrerror("allocation failure 2 in matrix()");
		m[i] -= ncl;
	}
	return m;
}

void free_matrix(double **m,int nrl,int nrh,int ncl,int nch)
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}


void free_vector(double *v, long nl, long nh)
/* free a float vector allocated with vector() */
{
	free((FREE_ARG) (v+nl-NR_END));
}

void covsrt(double **covar, int ma, int ia[], int mfit)
/* Expand in storage the covariance matrix covar, so as to take into account parameters that are
   being held fixed. (For the latter, return zero covariances.) */
{
  int i,j,k;
  double temp;
  for (i=mfit+1;i<=ma;i++)
    for (j=1;j<=i;j++) covar[i][j]=covar[j][i]=0.0;
  k=mfit;
  for (j=ma;j>=1;j--) 
  {
    if (ia[j]) 
    {
      for (i=1;i<=ma;i++) SWAP(covar[i][k],covar[i][j])
      for (i=1;i<=ma;i++) SWAP(covar[k][i],covar[j][i])
      k--;
    }
  }
}



void gaussj(double **a, int n, double **b, int m)
/* Linear equation solution by Gauss-Jordan elimination, equation (2.1.1) above. a[1..n][1..n]
   is the input matrix. b[1..n][1..m] is input containing the m right-hand side vectors. On
   output, a is replaced by its matrix inverse, and b is replaced by the corresponding set of solution
   vectors. */
{
  int *indxc,*indxr,*ipiv;
  int i,icol,irow,j,k,l,ll;
  double big,dum,pivinv,temp;
  ipiv=new int [n+2];
  indxc=new int [n+1];      // The integer arrays ipiv, indxr, and indxc are
  indxr=new int [n+1];      // used for bookkeeping on the pivoting.
  
  for (j=1;j<=n;j++) ipiv[j]=0;  
  for (i=1;i<=n;i++)   //This is the main loop over the columns to be reduced.
  {
    big=0.0;
    for (j=1;j<=n;j++)   //This is the outer loop of the search for a pivot element.
      if (ipiv[j] != 1)
	for (k=1;k<=n;k++) 
	{
	  if (ipiv[k] == 0) 
	  {
	    if (fabs(a[j][k]) >= big) 
	    {
	      big=fabs(a[j][k]);
	      irow=j;
	      icol=k;
	    }
	  }
	}
	++(ipiv[icol]);
	/* We now have the pivot element, so we interchange rows, if needed, to put the pivot
	 * element on the diagonal. The columns are not physically interchanged, only relabeled:
	 * indxc[i], the column of the ith pivot element, is the ith column that is reduced, while
	 * indxr[i] is the row in which that pivot element was originally located. If indxr[i] =
	 * indxc[i] there is an implied column interchange. With this form of bookkeeping, the
	 * solution b’s will end up in the correct order, and the inverse matrix will be scrambled
	 * by columns. */
	if (irow != icol) 
	{
	  for (l=1;l<=n;l++) SWAP(a[irow][l],a[icol][l])
          for (l=1;l<=m;l++) SWAP(b[irow][l],b[icol][l])
	}
	indxr[i]=irow;     // We are now ready to divide the pivot row by the
        indxc[i]=icol;     // pivot element, located at irow and icol.
        if (a[icol][icol] == 0.) nrerror("gaussj: Singular Matrix");
	pivinv=1.0/a[icol][icol];
        a[icol][icol]=1.0;
        for (l=1;l<=n;l++) a[icol][l] *= pivinv;
        for (l=1;l<=m;l++) b[icol][l] *= pivinv;
        for (ll=1;ll<=n;ll++)     // Next, we reduce the rows...
          if (ll != icol)         // ...except for the pivot one, of course.
	  {
	    dum=a[ll][icol];
	    a[ll][icol]=0.0;
            for (l=1;l<=n;l++) a[ll][l] -= a[icol][l]*dum;
            for (l=1;l<=m;l++) b[ll][l] -= b[icol][l]*dum;
	  }
  }
  /* This is the end of the main loop over columns of the reduction. It only remains to unscram-
     ble the solution in view of the column interchanges. We do this by interchanging pairs of
     columns in the reverse order that the permutation was built up. */
  for (l=n;l>=1;l--) 
  {
    if (indxr[l] != indxc[l])
      for (k=1;k<=n;k++)
	SWAP(a[k][indxr[l]],a[k][indxc[l]]);
  }    //And we are done.
  
    
  delete [] indxr; 
  delete [] indxc;  
  delete [] ipiv; 
}






void ssinus(double x, double a[], double *y, double dyda[], int na)
/*  y(x; a) is the sum of na/3 sin [ A*sin(2*M_PI*(nu*t+phi)) ]. The amplitude, frequency, and phase of the
    sin are stored in consecutive locations of a: a[i] = A_k , a[i+1] = nu_k , a[i+2] = phi_k ,
    k = 1, ..., na/3. The dimensions of the arrays are a[1..na], dyda[1..na] .*/
{
  int i;
  double fac,sinus,cosinus,arg;
  
  *y=0.0;
  for (i=1;i<=na-1;i+=3) 
  {  
    arg=2*M_PI*(a[i+1]*x+a[i+2]);
    sinus=sin(arg);
    cosinus=cos(arg);
    *y += a[i]*sinus;
    
    dyda[i]=sinus;
    dyda[i+1]=a[i]*cosinus*2*M_PI*x;
    dyda[i+2]=a[i]*cosinus*2*M_PI; //cout<<x<<" "<<arg<<" "<<*y<<" "<<dyda[i]<<" "<<dyda[i+1]<<" "<<dyda[i+2]<<endl;
  }
}



void mrqmin(double x[], double y[], double sig[], int ndata, double a[], int ia[],\
            int ma, double **covar, double **alpha, double *chisq,\
            void (*funcs)(double, double [], double *, double [], int), double *alamda)
/* Levenberg-Marquardt method, attempting to reduce the value χ2 of a fit between a set of data
   points x[1..ndata] , y[1..ndata] with individual standard deviations sig[1..ndata] ,
   and a nonlinear function dependent on ma coefficients a[1..ma]. The input array ia[1..ma]
   indicates by nonzero entries those components of a that should be fitted for, and by zero
   entries those components that should be held fixed at their input values. The program re-
   turns current best-fit values for the parameters a[1..ma], and χ2 = chisq. The arrays
   covar[1..ma][1..ma] , alpha[1..ma][1..ma] are used as working space during most
   iterations. Supply a routine funcs(x,a,yfit,dyda,ma) that evaluates the fitting function
   yfit, and its derivatives dyda[1..ma] with respect to the fitting parameters a at x. On
   the first call provide an initial guess for the parameters a, and set alamda<0 for initialization
   (which then sets alamda=.001 ). If a step succeeds chisq becomes smaller and alamda de-
   creases by a factor of 10. If a step fails alamda grows by a factor of 10. You must call this
   routine repeatedly until convergence is achieved. Then, make one final call with alamda=0, so
   that covar[1..ma][1..ma] returns the covariance matrix, and alpha the curvature matrix.
   (Parameters held fixed will return zero covariances.) */
{
  void covsrt(double **covar, int ma, int ia[], int mfit); 
  void gaussj(double **a, int n, double **b, int m); 
  void mrqcof(double x[], double y[], double sig[], int ndata, double a[],\
              int ia[], int ma, double **alpha, double beta[], double *chisq,\
              void (*funcs)(double, double [], double *, double [], int)); 
  int j,k,l;
  static int mfit;
  static double ochisq,*atry,*beta,*da,**oneda;  
  if (*alamda < 0.0)   //Initialization.
  {
    atry=new double [ma+1];
    beta=new double [ma+1];
    da=new double [ma+1];
    for (mfit=0,j=1;j<=ma;j++)
      if (ia[j]) mfit++;
    oneda=matrix(1,mfit,1,1);
    *alamda=0.001;
    mrqcof(x,y,sig,ndata,a,ia,ma,alpha,beta,chisq,funcs);
    ochisq=(*chisq);
    for (j=1;j<=ma;j++) 
      atry[j]=a[j];  
  } 
  for (j=1;j<=mfit;j++)    //Alter linearized fitting matrix, by augmenting diagonal elements.
  {
    for (k=1;k<=mfit;k++) covar[j][k]=alpha[j][k];

    covar[j][j]=alpha[j][j]*(1.0+(*alamda));
    oneda[j][1]=beta[j];
  }
  gaussj(covar,mfit,oneda,1); //Matrix solution.

  for (j=1;j<=mfit;j++) da[j]=oneda[j][1];
  if (*alamda == 0.0)     //Once converged, evaluate covariance matrix.
  {
    covsrt(covar,ma,ia,mfit);
    covsrt(alpha,ma,ia,mfit);   //Spread out alpha to its full size too.

    free_matrix(oneda,1,mfit,1,1);
    delete [] da;
    delete [] beta;
    delete [] atry;
    return;
  }
  for (j=0,l=1;l<=ma;l++)    //Did the trial succeed?
    if (ia[l])
      atry[l]=a[l]+da[++j];
  mrqcof(x,y,sig,ndata,atry,ia,ma,covar,da,chisq,funcs);
  if (*chisq < ochisq)    //Success, accept the new solution.
  {
    *alamda *= 0.1;
    ochisq=(*chisq);
    for (j=1;j<=mfit;j++) 
    {
      for (k=1;k<=mfit;k++) alpha[j][k]=covar[j][k];
      beta[j]=da[j];
    }
    for (l=1;l<=ma;l++) a[l]=atry[l];
  }
  else   //Failure, increase alamda and return.
  {
    *alamda *= 10.0;
    *chisq=ochisq;
  }
}





void mrqcof(double x[], double y[], double sig[], int ndata, double a[], int ia[],\
            int ma, double **alpha, double beta[], double *chisq,\
            void (*funcs)(double, double [], double *, double [], int))
/* Used by mrqmin to evaluate the linearized fitting matrix alpha, and vector beta as in (15.5.8),
   and calculate χ2 .  */
{
  int i,j,k,l,m,mfit=0;
  double ymod,wt,sig2i,dy,*dyda;

  dyda=new double [ma+1];
  for (j=1;j<=ma;j++)
    if (ia[j]) mfit++;
  for (j=1;j<=mfit;j++)   //Initialize (symmetric) alpha, beta.
  {
    for (k=1;k<=j;k++) alpha[j][k]=0.0;
    beta[j]=0.0;
  }
  *chisq=0.0;
  for (i=1;i<=ndata;i++)    //Summation loop over all data.
  {
    (*funcs)(x[i],a,&ymod,dyda,ma);
    sig2i=1.0/(sig[i]*sig[i]);
    dy=y[i]-ymod;
    for (j=0,l=1;l<=ma;l++) 
    {
      if (ia[l]) 
      {
	wt=dyda[l]*sig2i;
        for (j++,k=0,m=1;m<=l;m++)
	  if (ia[m]) alpha[j][++k] += wt*dyda[m];
        beta[j] += dy*wt;
      }
    }
    *chisq += dy*dy*sig2i;   //And find χ2 .
  }
  for (j=2;j<=mfit;j++)   //Fill in the symmetric side.
    for (k=1;k<j;k++) alpha[k][j]=alpha[j][k];
    delete [] dyda;
}




void avevar(double data[], unsigned long n, double *ave, double *var)
//Given array data[1..n], returns its mean as ave and its variance as var.
{
    unsigned long j;
    double s,ep; 
    for (*ave=0.0,j=1;j<=n;j++) *ave += data[j];
    *ave /= n;
    *var=ep=0.0; 
    for (j=1;j<=n;j++)
    {
        s=data[j]-(*ave);
        ep += s;
        *var += s*s;
    }
    *var=(*var-ep*ep/n)/(n-1); //Corrected two-pass formula (14.1.8).
    //cout<<"--- "<<*var<<endl;
}




void period(double x[], double y[], unsigned long n, double ofac, double hifac, double px[],\
             double py[], unsigned long np, unsigned long *nout, unsigned long *jmax, double *prob)
//Given n data points with abscissas x[1..n] (which need not be equally spaced) and ordinates
//y[1..n], and given a desired oversampling factor ofac (a typical value being 4 or larger),
//this routine fills array px[1..np] with an increasing sequence of frequencies (not angular
//580 Chapter 13. Fourier and Spectral Applications
//frequencies) up to hifac times the \average" Nyquist frequency, and fills array py[1..np]
//with the values of the Lomb normalized periodogram at those frequencies. The arrays x and y
//are not altered. np, the dimension of px and py, must be large enough to contain the output,
//or an error results. The routine also returns jmax such that py[jmax] is the maximum element
//in py, and prob, an estimate of the significance of that maximum against the hypothesis of
//random noise. A small value of prob indicates that a significant periodic signal is present.
{
    unsigned long i,j;
    double ave,c,cc,cwtau,effm,expy,pnow,pymax,s,ss,sumc,sumcy,sums,sumsh;
    double sumsy,swtau,var,wtau,xave,xdif,xmax,xmin,yy;
    double arg,wtemp,*wi,*wpi,*wpr,*wr;  
    try
    {
        wi=new double [n+1];
        wpi=new double [n+1];
        wpr=new double [n+1];
        wr=new double [n+1];
    }
    catch (std::bad_alloc)
    {
        nrerror("Brak pamieci");
    }
    *nout=0.5*ofac*hifac*n; 
    if (*nout > np) nrerror("output arrays too short in period");
    avevar(y,n,&ave,&var); //Get mean and variance of the input data.
    xmax=xmin=x[1]; //Go through data to get the range of abscissas
    for(j=1;j<=n;j++)
    {
        if (x[j] > xmax) xmax=x[j];
        if (x[j] < xmin) xmin=x[j];
    }
    xdif=xmax-xmin;
    xave=0.5*(xmax+xmin);
    pymax=0.0;
    pnow=1.0/(xdif*ofac); //Starting frequency.
    for (j=1;j<=n;j++)  //Initialize values for the trigonometric recurrences at each data point. The recurrences
    {                   //are done in double precision.
        arg=2*M_PI*((x[j]-xave)*pnow);
        wpr[j] = -2.0*sin(0.5*arg)*sin(0.5*arg);
        wpi[j]=sin(arg);
        wr[j]=cos(arg);
        wi[j]=wpi[j];
    }
//    for (i=1;i<=(*nout);i++) //Main loop over the frequencies to be evaluated.
    
    for (i=1;i<=*nout;i++) //Main loop over the frequencies to be evaluated.
    {
       // if(i%1000 == 0) cout<<i<<"  "<<(*nout)<<endl;
        px[i]=pnow;
        sumsh=sumc=0.0; //First, loop over the data to get tau and related quantities.
        
//        #pragma omp parallel for shared(n, wr, wi)\
        private(s, c, j)   default (none)\
        reduction(+: sumsh, sumc)\
        schedule (dynamic)
        for (j=1;j<=n;j++)
        { 
            c=wr[j];
            s=wi[j];
            sumsh += s*c;
            sumc += (c-s)*(c+s);
        } 
        //cout<<"kdsjfn"<<endl;
        
        wtau=0.5*atan2(2.0*sumsh,sumc);
        swtau=sin(wtau);
        cwtau=cos(wtau);
        sums=sumc=sumsy=sumcy=0.0; //Then, loop over the data again to get the periodogram value.
        
//        #pragma omp parallel for shared(n, wi, wr)\
        private(s, c, ss ,cc, yy, wtemp, j, y, wpr, wpi, cwtau, swtau, ave)   default (none)\
        reduction(+: sums, sumc, sumsy, sumcy)\
        schedule (dynamic)
        for (j=1;j<=n;j++)
        {
            s=wi[j];
            c=wr[j];
            ss=s*cwtau-c*swtau;
            cc=c*cwtau+s*swtau;
            sums += ss*ss;
            sumc += cc*cc;
            yy=y[j]-ave;
            sumsy += yy*ss;
            sumcy += yy*cc;
            wr[j]=((wtemp=wr[j])*wpr[j]-wi[j]*wpi[j])+wr[j]; //Update the trigonometric recurrences.
            wi[j]=(wi[j]*wpr[j]+wtemp*wpi[j])+wi[j];  
        } 
        
        py[i]=0.5*(sumcy*sumcy/sumc+sumsy*sumsy/sums)/var; 
        
        if (py[i] >= pymax ) pymax=py[(*jmax=i)];
        pnow += 1.0/(ofac*xdif); //The next frequency.
    }
    
    
    
    expy=exp(-pymax); //Evaluate statistical significance of the maximum.
    effm=2.0*(*nout)/ofac;
    *prob=effm*expy;
    if (*prob > 0.01) *prob=1.0-pow(1.0-expy,effm);
    delete [] wr;
    delete [] wpr;
    delete [] wpi;
    delete [] wi;
}



void lfit(double x[], double y[], double sig[], int ndat, double a[], int ia[],\
          int ma, double **covar, double *chisq, double freq[], void (*funcs)(double, double [], int, double []))
          /* Given a set of data points x[1..ndat], y[1..ndat] with individual standard deviations
             sig[1..ndat], use χ2 minimization to fit for some or all of the coefficients a[1..ma] of
             a function that depends linearly on a, y = i ai × afunci (x). The input array ia[1..ma]
             indicates by nonzero entries those components of a that should be fitted for, and by zero entries
             those components that should be held fixed at their input values. The program returns values
             for a[1..ma], χ2 = chisq, and the covariance matrix covar[1..ma][1..ma]. (Parameters
             held fixed will return zero covariances.) The user supplies a routine funcs(x,afunc,ma) that
             returns the ma basis functions evaluated at x = x in the array afunc[1..ma]. */
{
  int i,j,k,l,m,mfit=0;
  double ym,wt,sum,sig2i,**beta,*afunc;
  beta=matrix(1,ma,1,1);
  afunc=new double [ma+1];   
  for (j=1;j<=ma;j++)
    if (ia[j]) mfit++; 
  if (mfit == 0) nrerror("lfit: no parameters to be fitted");
  for (j=1;j<=mfit;j++)    //Initialize the (symmetric) matrix.
  {
    for (k=1;k<=mfit;k++) covar[j][k]=0.0;
    beta[j][1]=0.0;
  }  
  for (i=1;i<=ndat;i++)  // Loop over data to accumulate coefficients of the normal equations.
  {
    (*funcs)(x[i],afunc,ma,freq);
    ym=y[i];
    if (mfit < ma)    // Subtract off dependences on known pieces of the fitting function.
    {
      for (j=1;j<=ma;j++)
	if (!ia[j]) ym -= a[j]*afunc[j];
    }
    sig2i=1.0/(sig[i]*sig[i]);
    for (j=0,l=1;l<=ma;l++) 
    {
      if (ia[l]) 
      {
	wt=afunc[l]*sig2i;
        for (j++,k=0,m=1;m<=l;m++)
	  if (ia[m]) covar[j][++k] += wt*afunc[m];
	beta[j][1] += ym*wt;
      }
    }
  } 
  for (j=2;j<=mfit;j++)    // Fill in above the diagonal from symmetry.
    for (k=1;k<j;k++)
      covar[k][j]=covar[j][k];
  
  gaussj(covar,mfit,beta,1);  // Matrix solution.
  for (j=0,l=1;l<=ma;l++)
    if (ia[l]) a[l]=beta[++j][1];  // Partition solution to appropriate coefficientsa.
  *chisq=0.0;
  for (i=1;i<=ndat;i++) // Evaluate χ2 of the fit.
  {
    (*funcs)(x[i],afunc,ma,freq);
    for (sum=0.0,j=1;j<=ma;j++) sum += a[j]*afunc[j];
    *chisq += pow(((y[i]-sum)/sig[i]),2);
  }
  covsrt(covar,ma,ia,mfit);    // Sort covariance matrix to true order of fitting coefficients.
  delete [] afunc;
  free_matrix(beta,1,ma,1,1);
}



void linsin(double x, double afunc[], int ma, double freq[])
{
  int nfr;
  afunc[1] = 1.;
  nfr = (ma - 1)/2;
  for(int i=1; i<=nfr; i++)
  {
    afunc[2*i] = sin(2*M_PI*freq[i]*x);
    afunc[2*i+1] = cos(2*M_PI*freq[i]*x);
  }
}


void lindop(double x[], double y[], double sig[], int ndat, double a[],\
            double a_er[], int ma, double **covar, double *chisq)
{
  double *a1, *freq, *fi, *ampl, *eps;
  int *ia, i, j, k, l, *wskaznik;
  int mma=ma*2+1;
  double sum, var, calc, oc, sdev, rat, swag, DELA0, *DELfi, *DELa;
  
  a1=new double [mma+1];
  ia=new int [mma+1]; 
  freq=new double [ma+1];
  fi=new double [ma+1];
  ampl=new double [ma+1];
  eps=new double [mma+1];
  DELa=new double [ma+1];
  DELfi=new double [ma+1];
  
  for(int i=1; i<=mma; i++) ia[i]=1;
  for(i=2, j=1; i<=ma*3; i+=3, j++){  freq[j]=a[i];  }   //cout<<"---------------   "<<freq[j]<<endl;}
  
  lfit(x, y, sig, ndat, a1, ia, mma, covar, chisq, freq, linsin);
  
  for(i = 1; i<=ma; i++)
  {
    k = 2*i;
    l = k + 1;
    fi[i] = atan(a1[l]/a1[k]);
    if (a1[k] > 0 && a1[l] < 0) fi[i] += 2*M_PI;
    else if (a1[k] < 0) fi[i] += M_PI;
    ampl[i] = sqrt(a1[k]*a1[k] + a1[l]*a1[l]);
  }
  
  for(i=1, j=1; j<=ma; i+=3, j++)
  {
    a[i]=ampl[j];
    a[i+2]=fi[j]/(2*M_PI); //tutaj JDD faze liczy inaczej (z momentu maksimum) -- trzeba sie nad tym zastanowic
  }
  
   
  
  var = swag = 0.;
  for(int i=1; i<=ndat; i++)
  {
    sum = 0.;
    swag += 1/(sig[i]*sig[i]);
    for(int j=1; j<=ma*3; j+=3)
      sum +=  a[j]*sin(2*M_PI*(a[j+1]*x[i] + a[j+2]));
      
    calc = a1[1] + sum;
    oc = y[i] - calc;
    var = var + oc*oc;
  }  
  sdev = sqrt(var/double(ndat - 1));
  rat = double(ndat)/(swag*sdev*sdev); 
  
  for(int i=1; i<=mma; i++)
    for(int j=1;j <=mma; j++)     
                 covar[i][j] = covar[i][j]/rat;  

  for(int i=1; i<=mma; i++)  
  {eps[i] = sqrt(covar[i][i]); }
      
  for(int i=1; i<=ma; i++)  
  {
    k=2*i;
    l=k+1;
    DELfi[i] = sqrt( eps[l]*eps[l] + pow(fabs(a1[l]*eps[k]/a1[k]), 2))  /  (fabs(a1[k])*(1. + pow(fabs(a1[l]/a1[k]), 2)));
    DELa[i] =  sqrt(pow((a1[k]*eps[k]), 2) + pow((a1[l]*eps[l]), 2)) / ampl[i];  
  }
  DELA0 = eps[1];
  
  for(i=1, j=1; j<=ma; i+=3, j++)
  {
    a_er[i]=DELa[j]; 
    a_er[i+2]=DELfi[j]/(2*M_PI); 
  } 
  
  
  delete [] a1; delete [] ia; delete [] freq; delete [] fi; delete [] ampl; delete [] eps;
  delete [] DELa; delete [] DELfi;
}


void wybiel(double data[], double jasnosc_oryg[], double jasnosc_res[], double a[], int pik, int ndata)
{
  double arg, sum;
  stringstream licznikStr;
  string nazwa="wybiel";
  
  licznikStr<<fixed<<setw(3)<<setfill('0')<<setprecision(0)<<pik;
  nazwa+=licznikStr.str()+".dat";
  licznikStr.str("");
  ofstream out(nazwa.c_str());
  
  for(int i=1; i<=ndata; i++)
  {
    sum=0;
    for(int j=1; j<=pik*3; j+=3)
    {
      arg=2*M_PI*(a[j+1]*data[i]+a[j+2]);
      sum+=a[j]*sin(arg);
    }
    jasnosc_res[i] = jasnosc_oryg[i]-sum;
    out<<fixed<<setw(18)<<setprecision(11)<<data[i]<<fixed<<setw(22)<<setprecision(14)<<jasnosc_res[i]<<endl;
  }
  out.close();
}



void err_mrqmin(double x[], double y[], double sig[], int ndat, double a[],\
                double a_er[], int ma, double **covar)
{
  double *eps;  
  double sum, var, calc, oc, sdev, rat, swag;
  
  eps=new double [ma+1];
      
  
  var = swag = 0.;
  for(int i=1; i<=ndat; i++)
  {
    sum = 0.;
    swag += 1/(sig[i]*sig[i]);
    for(int j=1; j<=ma; j+=3)
      sum +=  a[j]*sin(2*M_PI*(a[j+1]*x[i] + a[j+2]));
      
    calc =  sum;
    oc = y[i] - calc;
    var = var + oc*oc;
  }  
  sdev = sqrt(var/double(ndat - 1));
  rat = double(ndat)/(swag*sdev*sdev);
  
  for(int i=1; i<=ma; i++)
    for(int j=1;j <=ma; j++)     
             covar[i][j] = covar[i][j]/rat;

  for(int i=1; i<=ma; i++)  
         eps[i] = sqrt(covar[i][i]);
      
  for(int i=1; i<=ma; i+=3)  
  {
    a_er[i]=eps[i];
    a_er[i+1]=eps[i+1];
    a_er[i+2]=eps[i+2];
  } 
  
  delete [] eps;
}


void rob_rys(int pik, double px[], double py[], int jmax, string plik)
{
  stringstream licznikStr;
  string nazwa="scargle", pol_sys;
  ofstream out;
  double minx, maxx;
  int wynik;
  
  minx=px[jmax]-2;
  maxx=px[jmax]+2;
  if(minx<0.0)
  {
    maxx-=minx;
    minx=0.0;
  }
  if(minx<0.01) minx=0.01;
  
  licznikStr<<fixed<<setw(3)<<setfill('0')<<setprecision(0)<<pik;
  nazwa+=licznikStr.str()+".eps";
  licznikStr.str("");
  out.open("rys.gnuplot");
  
  out<<"reset"<<endl;
  out<<"set terminal postscript enhanced color"<<endl;
  out<<"set terminal postscript landscape color  \"Helvetica\" 16 \\"<<endl;
  out<<"fontfile '/usr/share/texlive/texmf-dist/fonts/type1/public/amsfonts/cm/cmmi10.pfb' fontfile '/usr/share/texlive/texmf-dist/fonts/type1/public/amsfonts/cm/cmsy10.pfb' "<<endl;
  out<<"set output \""<<nazwa<<"\""<<endl;
  out<<"set xrange ["<<minx<<":"<<maxx<<"]"<<endl;
  out<<"plot \""<<plik<<"\" w l"<<endl;
  
  out.close();
 
  wynik=system("gnuplot rys.gnuplot");
}







void spec_wondow(double x[], double y[], unsigned long n)
{
    double nu0=0, dnu=1e-4, nu_max=24.5;
    double pn, ss, cc;
    
    
    ofstream out("spec_wondow.dat");
    
    while(1)
    {
        ss=cc=0;
        for(int i=0; i<n; i++)
        {
            ss+=sin(2*M_PI*nu0*x[i]);
            cc+=cos(2*M_PI*nu0*x[i]);
        }
        pn=(ss*ss+cc*cc)/n;
        
        
        out<<fixed<<setw(12)<<setprecision(8)<<nu0<<setw(12)<<setprecision(4)<<sqrt(4*pn/n)<<endl;
        
        if(nu0>=nu_max) break;
        nu0+=dnu;
    }
    out.close();
}
